import redis
import requests
from core.models import UserLoginSchema
from .models import PUser
from passlib.hash import pbkdf2_sha256
from core.models import User, DataCenterSchema
from core.auth import hash_pass
from fastapi import HTTPException
import random
import logging

logger = logging.getLogger(__name__)


def get_status(datacenter_url: str):
    logger.info(f"Health check datacenter '{datacenter_url}'")
    try:
        resp = requests.request(method="GET", url=datacenter_url, verify=False)
    except Exception:
        return False
    return int(resp.status_code / 100) == 2


r = redis.Redis(host="localhost", port=6379, db=0, decode_responses=True)


def reset():
    logging.info("Clean settings")
    r.set("datacenters_cnt", 1)
    all_keys = r.hgetall("datacenters_hash")
    if all_keys.keys():
        r.hdel("datacenters_hash", *list(all_keys.keys()))

    r.delete("rrlist")
    pipe = r.pipeline()
    for dc in all_keys.values():
        pipe.rpush("rrlist", dc)
    pipe.execute()


def update_status(datacenter):
    status = "OK" if get_status(datacenter_url=datacenter) is True else "Wrong"
    if status == "Wrong":
        logging.warning(f"{datacenter} is inactive status")
    r.hset("datacenter_status", datacenter, status)


def update_datacenter_status():
    logging.info("Update datacenters status")
    all_keys = r.hgetall("datacenters_hash")
    for dc in all_keys.values():
        update_status(datacenter=dc)


def add_datacenter(datacenter: str):
    cnt = int(r.get("datacenters_cnt"))
    r.hset("datacenters_hash", cnt, datacenter)
    r.incr("datacenters_cnt")
    r.set("roundrobin_cnt", 1)
    r.rpush("rrlist", datacenter)
    update_status(datacenter)


def rr_get():
    rr = r.rpoplpush("rrlist", "rrlist")
    return rr


def wr_get(centers: DataCenterSchema):
    cnt = int(r.get("datacenters_cnt"))
    indexis = range(1, cnt + 1)
    return random.choice(indexis, weights=())[0]


def set_token(username: str, datacenter: str, token: str):
    r.hset(f"tokens:{username}", datacenter, token)


def get_rtoken(username: str, datacenter: str):
    return r.hget(f"tokens:{username}", datacenter)


def get_rtokens(username: str):
    center_ids = list(r.hgetall("datacenters_hash").keys())
    tokens = []
    for id in center_ids:
        token = {}
        token["datacenter"] = r.hget("datacenters_hash", id)
        token["token"] = get_rtoken(username=username, datacenter=token["datacenter"])
        tokens.insert(0, token)
    return tokens


def get_datacenter_token(username: str, datacenter: str):
    return str(r.hget(f"tokens:{username}", datacenter))


def check_user(data: UserLoginSchema):
    user = PUser.find(PUser.username == data.username).all()
    if pbkdf2_sha256.verify(data.password, user[0].password):
        return user[0]
    return False


def create_puser(user: User):
    s = PUser.find(PUser.username == user.username).all()
    if s:
        raise HTTPException(status_code=403, detail="User already exists")
    user = PUser(
        username=user.username, email=user.email, password=hash_pass(user.password)
    )
    user.save()


def create_superpuser(user: User):
    s = PUser.find(PUser.username == user.username).all()
    if not s:
        puser = PUser(
            username=user.username,
            email=user.email,
            password=hash_pass(user.password),
            admin=1,
        )
        puser.save()
        return True
    return False
